import logging

class LogToPoDD(logging.Handler):

    def __init__(self,app_address, config, podd):

        logging.Handler.__init__(self)

        self.__address = app_address
        self.__podd=podd



        verbose = config["level"]
        if verbose == "DEBUG":
            level = logging.DEBUG
        elif verbose == "CRITICAL":
            level = logging.CRITICAL
        elif verbose == "ERROR":
            level = logging.ERROR
        elif verbose == "WARNING":
            level = logging.WARNING
        elif verbose == "INFO":
            level = logging.INFO
        else:
            level = logging.DEBUG

        self.setLevel(level)
        formatter = config["format"]
        self.setFormatter(formatter)



    def emit(self, record):  # override Handler's `emit` method

        try:

            self.__podd.send(str(self.format(record)))

        except: #drop that record
            pass







import logging
from multiprocessing import Queue
from logging.handlers import QueueHandler, QueueListener,RotatingFileHandler

class Cubelog():

    def __init__(self, __app_address, __log_formatter):

        self.__app_address=__app_address

        self.__log_formatter = logging.Formatter(__log_formatter)

        self.__cube_logger = logging.getLogger(self.__app_address)
        self.__cube_logger.setLevel(logging.DEBUG)

        self.log_queue = Queue(-1)  # local queue
        queue_handler = QueueHandler(self.log_queue)

        self.__cube_logger.addHandler(queue_handler)

        console_handler = logging.StreamHandler()
        console_handler.setFormatter(self.__log_formatter)
        console_handler.setLevel(logging.DEBUG)

        console_handler_DCWN= "CONSOLE"
        self.__handlers = {console_handler_DCWN:console_handler}

        self.__cube_logger.propagate = True




    def start(self):

        logging.getLogger(self.__app_address).info("CUBE LOGGING - Starting logging OPS")
        self.listener = QueueListener(self.log_queue, *self.__handlers.values(), respect_handler_level=True)
        self.listener.start()
        logging.getLogger(self.__app_address).info("CUBE LOGGING - Logging OPS is started for logging handlers: "+str(list(self.__handlers.keys())))

    def stop(self):

        logging.getLogger(self.__app_address).info("CUBE LOGGING - Stopping logging OPS for all handlers: "+str(list(self.__handlers.keys()))+" - End of logging.")
        self.listener.stop()

        for handler_dcwn, handler in self.__handlers.items():
            handler.flush()
            handler.close()




    def add_log_handlers(self, handlers):

        #check if new handlers
        new_handlers=True
        existing_dcwn=""
        for new_handler_dcwn in handlers.keys():
            if new_handler_dcwn in self.__handlers.keys():
                existing_dcwn=new_handler_dcwn
                new_handlers=False
                break

        if new_handlers:

            #stopping current listener and flush logs
            self.listener.stop()

            for handler_dcwn, handler in self.__handlers.items():
                handler.flush()

            #add new handlers
            for new_handler_dcwn, handler in handlers.items():
                self.__handlers[new_handler_dcwn]=handler
                logging.getLogger(self.__app_address).info("CUBE LOGGING - Added logging handler: " + str(new_handler_dcwn))

            #start listener
            self.listener = QueueListener(self.log_queue, *self.__handlers.values(), respect_handler_level=True)
            self.listener.start()
            logging.getLogger(self.__app_address).info(
                "CUBE LOGGING - Logging OPS is started for logging handlers: " + str(list(self.__handlers.keys())))

        else:
            logging.getLogger(self.__app_address).error("CUBE LOGGING - Failed to add logging handler: " + str(existing_dcwn)+", handler with the same DCWN already exist")
            raise ValueError("CUBE LOGGING - Failed to add logging handler: " + str(existing_dcwn)+", handler with the same DCWN already exist")


    #review
    def remove_log_handler(self, dcwn):

        try:
            handler=self.__handlers[dcwn]
        except KeyError:
            logging.getLogger(self.__app_address).error("CUBE LOGGING - Failed to remove logging handler: " + str(dcwn)+", handler does not exist")
            raise ValueError("CUBE LOGGING - Failed to remove logging handler: " + str(dcwn)+", handler does not exist")

        except Exception as e:
            logging.getLogger(self.__app_address).error("CUBE LOGGING - Unknown error when removing handler: " + str(dcwn)+", details: "+str(e))
            raise ValueError("CUBE LOGGING - Unknown error when removing handler: " + str(dcwn)+", details: "+str(e))

        handler.close()
        self.listener.stop()
        del self.__handlers[dcwn]
        # start listener
        self.listener = QueueListener(self.log_queue, *self.__handlers.values(), respect_handler_level=True)
        self.listener.start()
        logging.getLogger(self.__app_address).info("CUBE LOGGING - Deleted logging handler: " + str(dcwn))
        logging.getLogger(self.__app_address).info(
            "CUBE LOGGING - Logging OPS is started for logging handlers: " + str(list(self.__handlers.keys())))












from SchemaV.Schema import SpecError
from cube_business_ops.LogHandlers.LogToDisk import LogToDisk
from cube_business_ops.LogHandlers.LogToPoDD import LogToPoDD

class OPSManager():

    def __init__(self, configManager,app_address):


        self.__config_manager=configManager
        self.__app_address = app_address

    def validate_profile(self,application_config, params):

        self.__validated_bizops_conf= self.__config_manager.validate_profile(application_config, params)
        return self.__validated_bizops_conf


    def get_LogToDisk_handlers(self):

        handlers={}

        try: self.__validated_bizops_conf
        except : raise AssertionError("CUBE ERROR - Should validate BIZOPS profile before building log components")

        try: local_log_conf=self.__validated_bizops_conf["log"]["toDisk"]
        except: SpecError("SPEC ERROR - ToDisk logging section not found in profile")



        for log_conf in local_log_conf:

            if not log_conf["DCWN"] in handlers.keys():


                handlers[log_conf["DCWN"]]=LogToDisk(self.__app_address,log_conf)

            else:
                raise SpecError("SPEC ERROR - ToDisk DCWN: "+str(log_conf["DCWN"])+" should be unique")


        return handlers

    def get_stream_handlers(self):
        pass








    def get_monitor(self, metrics_podd, notification_pod):
        pass




import logging
import os



class Bizops_Checker():

    def __init__(self, address):
        self.__address = address
        self.__supported_watch_list = ["CPU", "RAM", "NETWORK", "PODD"]
        self.__supported_log_level=["DEBUG","INFO","WARNING","ERROR","CRITICAL"]

    def isString(self, x):
        return isinstance(x, str)

    def isInt(self, x):
        return isinstance(x, int)

    def check_log_format(self,x):
        try:
            formatter = logging.Formatter(x)
            return True
        except:
            logging.getLogger(self.__address).error("LOG FORMAT ERROR - " + "Log Formatter: "+str(x) + " is not correct")
            return False

    def isBoolean(self, x):
        return isinstance(x, bool)

    def isDirectory(self, dir):
        if os.path.isdir(dir):
            return True
        else:
            logging.getLogger(self.__address).error(
                "LOCAL LOG ERROR - " + str(dir) + ": is not a valid directory")
            return False


    def check_watch(self, params):

        to_watch = params.replace(' ', '').split(",")
        for param in to_watch:
            if not param in self.__supported_watch_list:
                logging.getLogger(self.__address).error("METRIC ERROR - "+str(param)+" metric is not supported")
                print(param)
                return False

        return True

    def check_local_log_level(self, verbose):

        if verbose in self.__supported_log_level:
            return True
        else:
            logging.getLogger(self.__address).error("LOCAL LOG ERROR - " + str(verbose) + ": verbosity level is not supported")
            return False


    def check_stream_log_level(self, verbose):

        if verbose in self.__supported_log_level:
            return True
        else:
            logging.getLogger(self.__address).error("STREAM LOG ERROR - " + str(verbose) + ": verbosity level is not supported")
            return False






import json
import os.path

from SchemaV.Checker import Checker
from SchemaV.Schema import Validator

from cube_business_ops.Checkers.Bizops_Checker import Bizops_Checker


class ConfigManager():

    def __init__(self, address):

        self.__address = address
        self.__properties_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "properties.json")
        self.__app_checker = Bizops_Checker(address)

    def __prepare_default_log_dir(self, default_log_dir):
        # can raise permission denied
        try:
            os.makedirs(default_log_dir, exist_ok=True)
            return default_log_dir
        except:
            raise

    def load_properties(self):

        try:

            with open(self.__properties_file) as propf:
                properties = json.load(propf)
            propf.close()

        except Exception as e:
            raise ImportError("BIZOPS PROPERTIES FILE IMPORT ERROR - file :" + str(
                self.__properties_file) + " is not json consistent or does not exist, details: " + str(e))

        try:

            self.__bizops_version = properties["bizops_version"]

            # ------------- application Data--------------
            self.__default_data_DCWN = properties["data"]["DCWN"]
            self.__default_data_podd_buffer_size = properties["data"]["podd_buffer_size"]

            # ------------- application Monitor --------------
            self.__default_monitor_DCWN = properties["monitor"]["DCWN"]

            # ------------- application Monitor Notifications --------------
            self.__default_notify_DCWN = properties["monitor"]["notify"]["DCWN"]
            self.__default_notify_podd_DCWN = properties["monitor"]["notify"]["podd_DCWN"]
            self.__default_notify_podd_buffer_size = properties["monitor"]["notify"]["podd_buffer_size"]

            # ------------- application Monitor Metrics --------------
            self.__default_metrics_DCWN = properties["monitor"]["metrics"]["DCWN"]
            self.__default_metrics_podd_DCWN = properties["monitor"]["metrics"]["podd_DCWN"]
            self.__default_metrics_heartbeat = properties["monitor"]["metrics"]["heartbeat"]
            self.__default_metrics_watch = properties["monitor"]["metrics"]["watch"]
            self.__default_metric_podd_buffer_size = properties["monitor"]["metrics"]["podd_buffer_size"]

            # ------------- application Log --------------
            self.__default_log_DCWN = properties["log"]["DCWN"]

            # ------------- application Log Local On Disk --------------
            self.__default_local_log_DCWN = properties["log"]["toDisk"]["DCWN"]
            self.__default_local_log_level = properties["log"]["toDisk"]["logging-level"]
            self.__default_max_log_files = properties["log"]["toDisk"]["max-log-count"]
            self.__default_max_log_size = properties["log"]["toDisk"]["max-log-size"]
            self.__default_log_dir = properties["log"]["toDisk"]["logging-dir"]
            self.__default_local_log_format=properties["log"]["toDisk"]["logging-format"]

            # ------------- application Log Stream--------------
            self.__default_stream_log_DCWN = properties["log"]["stream"]["DCWN"]
            self.__default_stream_log_podd_DCWN = properties["log"]["stream"]["podd_DCWN"]
            self.__default_stream_log_level = properties["log"]["stream"]["logging-level"]
            self.__default_stream_log_podd_buffer_size = properties["log"]["stream"]["podd_buffer_size"]
            self.__default_stream_log_format = properties["log"]["stream"]["logging-format"]


        except Exception as e:
            raise ImportError(
                "BIZOPS PROPERTIES FILE IMPORT ERROR - In file :" + str(self.__properties_file) + ", details: " + str(
                    e))

    def get_app_schema(self, params):
        """

        :param params: params["verify_reachable"]: True/False
        :return:
        """

        try:
            self.__group_DCWN = params["group-DCWN"]
        except:
            raise AssertionError("BUSINESS PROFILE IMPORT ERROR - Cannot import schema without a defined Business Group Name")

        # default Topics & Keys

        self.__default_notification_topic = str(self.__group_DCWN) + "_NOTIFICATIONS"
        self.__default_metrics_topic = str(self.__group_DCWN) + "_METRICS"
        self.__default_streamlog_topic = str(self.__group_DCWN) + "_LOG"

        self.__default_data_podd_key = "None"
        self.__default_notification_podd_key = self.__address
        self.__default_metrics_podd_key = self.__address
        self.__default_streamlog_podd_key = self.__address

        schema = {'checkIn': None, 'checkNext': {
            'version': {'checkIn': Checker(False, self.__app_checker.isString, "NA"), 'checkNext': None},
            'desc': {'checkIn': Checker(False, self.__app_checker.isString, "NA"), 'checkNext': None},

            'name': {'checkIn': Checker(False, self.__app_checker.isString, "NA"), 'checkNext': None},
            'DCWN': {'checkIn': Checker(True, self.__app_checker.isString, None), 'checkNext': None},

            # region: used to force the app deployment in a specified region, NA for any where
            'region': {'checkIn': Checker(None, self.__app_checker.isString, "NA"), 'checkNext': None},

            'data': {'checkIn': None, 'checkNext': {

                'DCWN': {'checkIn': Checker(False, self.__app_checker.isString, self.__default_data_DCWN),
                         'checkNext': None},

                'podd': {'checkIn': None, 'checkNext': [{'checkIn': None, 'checkNext': {

                    'name': {'checkIn': Checker(True, self.__app_checker.isString, None), 'checkNext': None},
                    'DCWN': {'checkIn': Checker(True, self.__app_checker.isString, None), 'checkNext': None},
                    'network-plane': {'checkIn': Checker(True, self.__app_checker.isString, None), 'checkNext': None},
                    'topic': {'checkIn': Checker(True, self.__app_checker.isString, None), 'checkNext': None},
                    'key': {'checkIn': Checker(False, self.__app_checker.isString, self.__default_data_podd_key),
                            'checkNext': None},
                    'buffer': {
                        'checkIn': Checker(False, self.__app_checker.isInt, self.__default_data_podd_buffer_size),
                        'checkNext': None}}}]}}},

            'monitor': {'checkIn': Checker(True, None, None), 'checkNext': {

                'DCWN': {'checkIn': Checker(False, self.__app_checker.isString, self.__default_monitor_DCWN),
                         'checkNext': None},
                'notify': {'checkIn': Checker(True, None, None), 'checkNext': {

                    'DCWN': {'checkIn': Checker(False, self.__app_checker.isString, self.__default_notify_DCWN),
                             'checkNext': None},

                    'podd': {'checkIn': Checker(True, None, None), 'checkNext': {

                        'name': {'checkIn': Checker(False, self.__app_checker.isString, "NA"), 'checkNext': None},
                        'DCWN': {
                            'checkIn': Checker(False, self.__app_checker.isString, self.__default_notify_podd_DCWN),
                            'checkNext': None},
                        'network-plane': {'checkIn': Checker(True, self.__app_checker.isString, None),
                                          'checkNext': None},
                        'topic': {
                            'checkIn': Checker(False, self.__app_checker.isString, self.__default_notification_topic),
                            'checkNext': None},
                        'key': {'checkIn': Checker(False, self.__app_checker.isString,
                                                   self.__default_notification_podd_key), 'checkNext': None},
                        'buffer': {
                            'checkIn': Checker(False, self.__app_checker.isInt, self.__default_notify_podd_buffer_size),
                            'checkNext': None}}}}},

                'metrics': {'checkIn': Checker(True, None, None), 'checkNext': {

                    'DCWN': {'checkIn': Checker(False, self.__app_checker.isString, self.__default_metrics_DCWN),
                             'checkNext': None},
                    'heartbeat': {'checkIn': Checker(False, self.__app_checker.isInt, self.__default_metrics_heartbeat),
                                  'checkNext': None},
                    'watch': {'checkIn': Checker(False, self.__app_checker.check_watch, self.__default_metrics_watch),
                              'checkNext': None},

                    'podd': {'checkIn': Checker(True, None, None), 'checkNext': {

                        'name': {'checkIn': Checker(False, self.__app_checker.isString, "NA"), 'checkNext': None},
                        'DCWN': {
                            'checkIn': Checker(False, self.__app_checker.isString, self.__default_metrics_podd_DCWN),
                            'checkNext': None},
                        'network-plane': {'checkIn': Checker(True, self.__app_checker.isString, None),
                                          'checkNext': None},
                        'topic': {'checkIn': Checker(False, self.__app_checker.isString, self.__default_metrics_topic),
                                  'checkNext': None},
                        'key': {'checkIn': Checker(False, self.__app_checker.isString, self.__default_metrics_podd_key),
                                'checkNext': None},
                        'buffer': {
                            'checkIn': Checker(False, self.__app_checker.isInt, self.__default_metric_podd_buffer_size),
                            'checkNext': None}}}}}}},

            'log': {'checkIn': Checker(True, None, None), 'checkNext': {

                'DCWN': {'checkIn': Checker(False, self.__app_checker.isString, self.__default_log_DCWN),
                         'checkNext': None},

                'toDisk': {'checkIn': Checker(False, None, []), 'checkNext': [{
                     "checkIn":Checker(False, None, {}),
                     "checkNext":{

                    'logging-level': {'checkIn': Checker(False, self.__app_checker.check_local_log_level,
                                                   self.__default_local_log_level), 'checkNext': None},

                    'logging-format':  {'checkIn': Checker(False, self.__app_checker.check_log_format,
                                                   self.__default_local_log_format), 'checkNext': None},

                    'DCWN': {'checkIn': Checker(False, self.__app_checker.isString, self.__default_local_log_DCWN),
                             'checkNext': None},
                    'logging-dir': {'checkIn': Checker(False, self.__app_checker.isDirectory,
                                                     self.__prepare_default_log_dir(self.__default_log_dir)),
                                  'checkNext': None},
                    'max-log-count': {'checkIn': Checker(False, self.__app_checker.isInt, self.__default_max_log_files),
                                      'checkNext': None},
                    'max-log-size': {'checkIn': Checker(False, self.__app_checker.isInt, self.__default_max_log_size),
                                     'checkNext': None}
                }}]
                           },

                'stream': {'checkIn': Checker(True, None, None), 'checkNext': [{
                    "checkIn": Checker(True, None, None),
                    "checkNext": {

                    'logging-level': {'checkIn': Checker(False, self.__app_checker.check_stream_log_level,
                                                   self.__default_stream_log_level), 'checkNext': None},

                    'logging-format': {'checkIn': Checker(False, self.__app_checker.check_log_format,
                                                      self.__default_stream_log_format), 'checkNext': None},

                    'DCWN': {'checkIn': Checker(False, self.__app_checker.isString, self.__default_stream_log_DCWN),
                             'checkNext': None},

                    'podd': {'checkIn': Checker(True, None, None), 'checkNext': {

                        'name': {'checkIn': Checker(False, self.__app_checker.isString, "NA"), 'checkNext': None},
                        'DCWN': {
                            'checkIn': Checker(False, self.__app_checker.isString, self.__default_stream_log_podd_DCWN),
                            'checkNext': None},
                        'network-plane': {'checkIn': Checker(True, self.__app_checker.isString, None),
                                          'checkNext': None},
                        'topic': {
                            'checkIn': Checker(False, self.__app_checker.isString, self.__default_streamlog_topic),
                            'checkNext': None},
                        'key': {
                            'checkIn': Checker(False, self.__app_checker.isString, self.__default_streamlog_podd_key),
                            'checkNext': None},
                        'buffer': {'checkIn': Checker(False, self.__app_checker.isInt,
                                                      self.__default_stream_log_podd_buffer_size), 'checkNext': None}}}
                }}]
                           }
            }
                    }}}

        return schema

    def validate_profile(self, application_config, params):

        app_validator = Validator(self.get_app_schema(params))
        return app_validator.validate(application_config)



from logging.handlers import RotatingFileHandler
import logging

import os

class LogToDisk(RotatingFileHandler):

    def __init__(self, app_address, config):

        self.__address=app_address

        verbose = config["logging-level"]
        log_file=os.path.join(config["logging-dir"], app_address+"-"+str(verbose)+".log")
        max_log_count=config["max-log-count"]
        max_log_size_bytes=config["max-log-size"]*1048576

        RotatingFileHandler.__init__(self,log_file, maxBytes=max_log_size_bytes, backupCount=max_log_count)


        if verbose == "DEBUG":
            level = logging.DEBUG
        elif verbose == "CRITICAL":
            level = logging.CRITICAL
        elif verbose == "ERROR":
            level = logging.ERROR
        elif verbose == "WARNING":
            level = logging.WARNING
        elif verbose == "INFO":
            level = logging.INFO
        else:
            level = logging.DEBUG

        self.setLevel(level)
        formatter = logging.Formatter(config["logging-format"])
        self.setFormatter(formatter)


